import { Fragment, useEffect, useState } from "react";
import { UserResource, UsersResource } from "../Resources";
import { useErrorBoundary } from "react-error-boundary";
import { useLoginContext } from "../LoginContext";
import { deleteUser, getUsers } from "../backend/userapi";
import LoadingIndicator from "./LoadingIndicator";
import { Button, ListGroup } from "react-bootstrap";
import AddUserDialog from "./AddUserDialog";
import EditUserDialog from "./EditUserDialog";

export default function PageUsermanagment() {
    
    const [users, setUsers] = useState<UsersResource>();
    const [showDialog, setShowDialog] = useState(false);
    const [showEditDialog, setShowEditDialog] = useState(false);
    const { showBoundary } = useErrorBoundary();
    const [selectedUser, setSelectedUser] = useState<UserResource | null>({} as UserResource);
    const { loginInfo } = useLoginContext();

    function show() {
      setShowDialog(true);
    }
    function hide() {
      setShowDialog(false);
    }

    function showEdit(user : UserResource) {
      setSelectedUser(user);
      setShowEditDialog(true);
    }
    function hideEdit() {
      setSelectedUser(null);
      setShowEditDialog(false);
    }

    async function load(){
        try{
            const c = await getUsers();
            setUsers(c);
        }catch(err){
            showBoundary(err);
        }
    }

    async function doDeleteUser(userId:string){
      try{
        await deleteUser(userId);
        load();
      }catch(err){
        showBoundary(err);
      }
    }

    useEffect(() => {
        load()
      }, []);
    
      if(!users){
        return <LoadingIndicator />
      }else{
        return (
            <div>
                <h2 className="ms-3 mt-3">User Management</h2>
                <Button variant="warning" className="addUserButton ms-3 mb-3" onClick={show}>Add Account</Button>
                <ListGroup className="listUsers">
                  {users.users.map((user) => (
                    <ListGroup.Item className={`user-${user.name}`} key={user.id}>
                      <div className="d-flex justify-content-between align-items-center">
                        <div>
                          {user.admin ? "Admin" : "User"} <strong>{user.name}</strong> - {user.email}
                        </div>
                        <div>
                          {!user.admin 
                            ? 
                            <>
                            <Button className={`editButton-${user.name} me-2`} variant="primary" onClick={() =>showEdit(user)}>Edit</Button>
                            <Button className={`deleteButton-${user.name}`} variant="danger" onClick={()=>doDeleteUser(user.id!)}>Delete</Button></>
                            : null}
                        </div>
                      </div>
                    </ListGroup.Item>
                  ))}
                </ListGroup>
                <Fragment>
                  <AddUserDialog show={showDialog} onHide={hide} />
                </Fragment>
                <Fragment>
                  <EditUserDialog show={showEditDialog} onHide={hideEdit} selectedUser={selectedUser!} />
                </Fragment>
            </div>
        );
      }
}